@@ -153,6 +153,7 @@ class KollaAnsibleMixin(object):
 
 
 def _split_hook_sequence_number(hook):
+    hook = os.path.basename(hook)
     parts = hook.split("-", 1)
     if len(parts) < 2:
         return (DEFAULT_SEQUENCE_NUMBER, hook)
@@ -181,22 +182,38 @@ class HookDispatcher(CommandHook):
     def get_parser(self, prog_name):
         pass
 
-    def _find_hooks(self, config_path, target):
+    def _find_hooks(self, env_paths, target):
         name = self.name
-        path = os.path.join(config_path, "hooks", name, "%s.d" % target)
-        self.logger.debug("Discovering hooks in: %s" % path)
-        if not os.path.exists:
-            return []
-        hooks = glob.glob(os.path.join(path, "*.yml"))
+        # Map from hook directory path to a set of hook basenames in that path.
+        hooks: {str: {str}} = {}
+        for env_path in env_paths:
+            path = os.path.join(env_path, "hooks", name, "%s.d" % target)
+            self.logger.debug("Discovering hooks in: %s" % path)
+            if not os.path.exists(path):
+                continue
+
+            hook_paths = glob.glob(os.path.join(path, "*.yml"))
+            hook_basenames = {os.path.basename(hook) for hook in hook_paths}
+
+            # Override any earlier hooks with the same basename.
+            for other_hooks in hooks.values():
+                other_hooks -= hook_basenames
+
+            hooks[path] = hook_basenames
+
+        # Return a flat list of hook paths (including directory).
+        hooks = [os.path.join(path, basename)
+                 for path, basenames in hooks.items()
+                 for basename in basenames]
         self.logger.debug("Discovered the following hooks: %s" % hooks)
         return hooks
 
-    def hooks(self, config_path, target, filter):
+    def hooks(self, env_paths, target, filter):
         hooks_out = []
         if filter == "all":
             self.logger.debug("Skipping all hooks")
             return hooks_out
-        hooks_in = self._find_hooks(config_path, target)
+        hooks_in = self._find_hooks(env_paths, target)
         # Hooks can be prefixed with a sequence number to adjust running order,
         # e.g 10-my-custom-playbook.yml. Sort by sequence number.
         hooks_in = sorted(hooks_in, key=_split_hook_sequence_number)
@@ -210,8 +227,12 @@ class HookDispatcher(CommandHook):
         return hooks_out
 
     def run_hooks(self, parsed_args, target):
-        config_path = parsed_args.config_path
-        hooks = self.hooks(config_path, target, parsed_args.skip_hooks)
+        env_paths = [parsed_args.config_path]
+        environment_finder = utils.EnvironmentFinder(
+            parsed_args.config_path, parsed_args.environment)
+        env_paths.extend(environment_finder.ordered_paths())
+
+        hooks = self.hooks(env_paths, target, parsed_args.skip_hooks)
         if hooks:
             self.logger.debug("Running hooks: %s" % hooks)
             self.command.run_kayobe_playbooks(parsed_args, hooks)
@@ -373,6 +394,11 @@ class KollaAnsibleRun(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
 
     def take_action(self, parsed_args):
         self.app.LOG.debug("Running Kolla Ansible command")
+
+        # First prepare configuration.
+        self.generate_kolla_ansible_config(parsed_args)
+
+        # Run the kolla-ansible command.
         self.run_kolla_ansible(parsed_args, parsed_args.command,
                                parsed_args.kolla_inventory_filename)
 
@@ -704,7 +730,9 @@ class SeedServiceDeploy(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
         self.app.LOG.debug("Deploying seed services")
         playbooks = _build_playbook_list(
             "seed-deploy-containers")
-        self.run_kayobe_playbooks(parsed_args, playbooks)
+        extra_vars = {"kayobe_action": "deploy"}
+        self.run_kayobe_playbooks(parsed_args, playbooks,
+                                  extra_vars=extra_vars)
         self.generate_kolla_ansible_config(parsed_args, service_config=False,
                                            bifrost_config=True)
 
@@ -713,7 +741,48 @@ class SeedServiceDeploy(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
             "seed-credentials",
             "seed-introspection-rules",
             "dell-switch-bmp")
-        self.run_kayobe_playbooks(parsed_args, playbooks)
+        self.run_kayobe_playbooks(parsed_args, playbooks,
+                                  extra_vars=extra_vars)
+
+
+class SeedServiceDestroy(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
+                         Command):
+    """Destroy the seed services.
+
+    * Destroys user defined containers
+    * Destroys kolla deployed containers
+    * Destroys docker registry
+    """
+
+    def take_action(self, parsed_args):
+        if not parsed_args.yes_i_really_really_mean_it:
+            self.app.LOG.error("This will permanently destroy all services "
+                               "and data. Specify "
+                               "--yes-i-really-really-mean-it to confirm that "
+                               "you understand this.")
+            sys.exit(1)
+        self.app.LOG.debug("Destroying seed services")
+        self.generate_kolla_ansible_config(parsed_args, service_config=False,
+                                           bifrost_config=False)
+        extra_args = ["--yes-i-really-really-mean-it"]
+        self.run_kolla_ansible_seed(parsed_args, "destroy",
+                                    extra_args=extra_args)
+
+        extra_vars = {"kayobe_action": "destroy"}
+        playbooks = _build_playbook_list(
+            "seed-deploy-containers",
+            "docker-registry")
+        self.run_kayobe_playbooks(parsed_args, playbooks,
+                                  extra_vars=extra_vars)
+
+    def get_parser(self, prog_name):
+        parser = super(SeedServiceDestroy, self).get_parser(prog_name)
+        group = parser.add_argument_group("Services")
+        group.add_argument("--yes-i-really-really-mean-it",
+                           action='store_true',
+                           help="confirm that you understand that this will "
+                                "permanently destroy all services and data.")
+        return parser
 
 
 class SeedServiceUpgrade(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
@@ -736,7 +805,9 @@ class SeedServiceUpgrade(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
         self.app.LOG.debug("Upgrading seed services")
         playbooks = _build_playbook_list(
             "seed-deploy-containers")
-        self.run_kayobe_playbooks(parsed_args, playbooks)
+        extra_vars = {"kayobe_action": "deploy"}
+        self.run_kayobe_playbooks(parsed_args, playbooks,
+                                  extra_vars=extra_vars)
         self.generate_kolla_ansible_config(parsed_args, service_config=False,
                                            bifrost_config=True)
 
@@ -1327,7 +1398,7 @@ class OvercloudServiceConfigurationValidate(KayobeAnsibleMixin,
         group = parser.add_argument_group("Service Configuration Validate")
         group.add_argument("--output-dir", required=True,
                            help="the directory to store the results of running"
-                                "the config validator (required)")
+                                " the config validator (required)")
         return parser
 
     def take_action(self, parsed_args):
@@ -1833,6 +1904,15 @@ class NetworkConnectivityCheck(KayobeAnsibleMixin, VaultMixin, Command):
         self.run_kayobe_playbooks(parsed_args, playbooks)
 
 
+class BaremetalComputeRegister(KayobeAnsibleMixin, VaultMixin, Command):
+    """Register baremetal compute nodes in Ironic."""
+
+    def take_action(self, parsed_args):
+        self.app.LOG.debug("Register baremetal compute nodes in Ironic.")
+        playbooks = _build_playbook_list("baremetal-compute-register")
+        self.run_kayobe_playbooks(parsed_args, playbooks)
+
+
 class BaremetalComputeInspect(KayobeAnsibleMixin, VaultMixin, Command):
     """Perform hardware inspection on baremetal compute nodes."""
 
@@ -1843,6 +1923,42 @@ class BaremetalComputeInspect(KayobeAnsibleMixin, VaultMixin, Command):
         self.run_kayobe_playbooks(parsed_args, playbooks)
 
 
+class BaremetalComputeIntrospectionDataSave(KayobeAnsibleMixin, VaultMixin,
+                                            Command):
+    """Save hardware introspection data for the baremetal compute nodes.
+
+    Save hardware introspection data from the overcloud's ironic inspector
+    service to the Ansible control host.
+    """
+
+    def get_parser(self, prog_name):
+        parser = super(BaremetalComputeIntrospectionDataSave, self).get_parser(
+            prog_name)
+        group = parser.add_argument_group("Introspection data")
+        # Defaults for these are applied in the playbook.
+        group.add_argument("--output-dir", type=str,
+                           help="Path to directory in which to save "
+                                "introspection data. Default: "
+                                "$PWD/baremetal-compute-introspection-data")
+        group.add_argument("--output-format", type=str,
+                           help="Format in which to save output data. One of "
+                                "JSON or YAML. Default: JSON",
+                           choices=["JSON", "YAML"])
+        return parser
+
+    def take_action(self, parsed_args):
+        self.app.LOG.debug("Saving introspection data")
+        extra_vars = {}
+        if parsed_args.output_dir:
+            extra_vars['output_dir'] = parsed_args.output_dir
+        if parsed_args.output_format:
+            extra_vars['output_format'] = parsed_args.output_format
+        playbooks = _build_playbook_list(
+            "baremetal-compute-introspection-data-save")
+        self.run_kayobe_playbooks(parsed_args, playbooks,
+                                  extra_vars=extra_vars)
+
+
 class BaremetalComputeManage(KayobeAnsibleMixin, VaultMixin, Command):
     """Put baremetal compute nodes into the manageable provision state."""
 
