@@ -95,10 +95,6 @@ class ActionModule(ActionBase):
             # Get the network interface for this network.
             iface = ("{{ '%s' | net_interface }}" % net_name)
             iface = self._templar.template(iface)
-            if iface:
-                # Ansible fact names replace dashes with underscores.
-                # FIXME(mgoddard): Is this still required?
-                iface = iface.replace('-', '_')
             if required and not iface:
                 msg = ("Required network '%s' (%s) does not have an interface "
                        "configured for this host" % (net_name, description))
@@ -154,7 +150,10 @@ class ActionModule(ActionBase):
             # For a bridge, use a veth pair connected to the bridge. Otherwise
             # use the interface directly.
             if is_bridge:
-                external_interface = patch_prefix + interface + patch_suffix
+                # interface names can't be longer than 15 characters
+                char_limit = 15 - len(patch_prefix) - len(patch_suffix)
+                external_interface = patch_prefix + interface[:char_limit] + \
+                    patch_suffix
             else:
                 external_interface = interface
             neutron_external_interfaces.append(external_interface)
