@@ -13,7 +13,9 @@
 # under the License.
 
 import base64
+from collections import defaultdict
 import glob
+import graphlib
 from importlib.metadata import Distribution
 import json
 import logging
@@ -22,6 +24,7 @@ import shutil
 import subprocess
 import sys
 
+from ansible.parsing.yaml.loader import AnsibleLoader
 import yaml
 
 from kayobe import exception
@@ -88,7 +91,7 @@ def _get_base_path():
         return os.path.join(prefix, "share", "kayobe")
 
     # Assume uninstalled
-    return os.path.join(os.path.realpath(__file__), "..")
+    return os.path.join(os.path.dirname(os.path.realpath(__file__)), "..")
 
 
 def galaxy_role_install(role_file, roles_path, force=False):
@@ -151,11 +154,28 @@ def read_yaml_file(path):
     try:
         content = read_file(path)
     except IOError as e:
-        print("Failed to open config dump file %s: %s" %
+        print("Failed to open YAML file %s: %s" %
               (path, repr(e)))
         sys.exit(1)
     try:
         return yaml.safe_load(content)
+    except yaml.YAMLError as e:
+        print("Failed to decode YAML file %s: %s" %
+              (path, repr(e)))
+        sys.exit(1)
+
+
+def read_config_dump_yaml_file(path):
+    """Read and decode a configuration dump YAML file."""
+    try:
+        content = read_file(path)
+    except IOError as e:
+        print("Failed to open config dump file %s: %s" %
+              (path, repr(e)))
+        sys.exit(1)
+    try:
+        # AnsibleLoader supports loading vault encrypted variables.
+        return AnsibleLoader(content).get_single_data()
     except yaml.YAMLError as e:
         print("Failed to decode config dump YAML file %s: %s" %
               (path, repr(e)))
@@ -259,7 +279,7 @@ def intersect_limits(args_limit, cli_limit):
         separator = ',&'
     else:
         separator = ':&'
-    limits = [l for l in [args_limit, cli_limit] if l]
+    limits = [limit for limit in [args_limit, cli_limit] if limit]
     return separator.join(limits)
 
 
@@ -287,3 +307,99 @@ def copy_dir(src, dest, exclude=None):
             copy_dir(src_path, dest_path)
         else:
             shutil.copy2(src_path, dest_path)
+
+
+def get_kayobe_environment_path(base_path, environment):
+    """Return the path to the Kayobe environment or None if not specified."""
+    env_path = None
+    if environment:
+        # Specified via --environment or KAYOBE_ENVIRONMENT.
+        kc_environments = os.path.join(base_path, "environments")
+        env_path = os.path.join(kc_environments, environment)
+    return env_path
+
+
+class EnvironmentFinder(object):
+    """Dependency resolver for kayobe environments
+
+    The constraints are specified via a .kayobe-environment file.
+    """
+
+    def __new__(cls, base_path, environment):
+        # Singleton instance so we don't have to resolve dependencies multiple
+        # times or pass round a single instance.
+        it = cls.__dict__.get("__it__")
+        if it is None:
+            it = {}
+        if (base_path, environment) in it:
+            return it[(base_path, environment)]
+        singleton = object.__new__(cls)
+        singleton._init(base_path, environment)
+        it[(base_path, environment)] = singleton
+        return singleton
+
+    def _init(self, base_path, environment):
+        self._base_path = base_path
+        self._environment = environment
+        self._ordering = None
+
+    @staticmethod
+    def _read_metadata(path):
+        if os.path.exists(path) and os.path.isfile(path):
+            metadata = read_yaml_file(path)
+            return metadata
+        return {}
+
+    def _collect(self, environment, result, visited):
+        # Updates result to contain dependency graph
+        base = self._base_path
+        env_path = os.path.join(base, 'environments', environment)
+        dot_environment_path = os.path.join(env_path, '.kayobe-environment')
+        if dot_environment_path in visited:
+            return
+        visited.add(dot_environment_path)
+        metadata = EnvironmentFinder._read_metadata(dot_environment_path)
+        dependencies = metadata.get("dependencies", [])
+        if not isinstance(dependencies, list):
+            raise exception.Error(".kayobe-environment: dependencies field "
+                                  "should be a list")
+        result[environment] |= set(dependencies)
+        for dependency in dependencies:
+            if not isinstance(dependency, str):
+                raise exception.Error("Kayobe environment dependency items "
+                                      "should be strings")
+            self._collect(dependency, result, visited)
+
+    def ordered(self):
+        """List of environments ordered by the constraints"""
+        environment = self._environment
+        if not environment:
+            return []
+        if self._ordering is not None:
+            return self._ordering.copy()
+        graph = defaultdict(set)
+        self._collect(environment, graph, set())
+        ts = graphlib.TopologicalSorter(graph)
+        try:
+            ordering = list(ts.static_order())
+        except graphlib.CycleError as e:
+            # https://docs.python.org/3/library/graphlib.html#graphlib.CycleError
+            cycle = e.args[1]
+            raise exception.Error("You have created a cycle with your "
+                                  "environment dependencies. Please break "
+                                  "this cycle and try again. The cycle is: %s"
+                                  % cycle)
+        self._ordering = ordering if ordering else [environment]
+        return self._ordering.copy()
+
+    def ordered_paths(self):
+        """Paths to each environment ordered by the constraints"""
+        result = []
+        environments = self.ordered()
+        for environment in environments:
+            full_path = get_kayobe_environment_path(
+                self._base_path,
+                environment
+            )
+            result.append(full_path)
+        return result
